/*
 * config.h - MAP Controller configuration header file
 *
 * Copyright (C) 2020 IOPSYS Software Solutions AB. All rights reserved.
 *
 * Author: anjan.chanda@iopsys.eu
 *
 */

#ifndef CONFIG_H
#define CONFIG_H

/* defined in main.c */
extern int verbose;

enum cred_diff {
	CONFIG_DIFF_BASE             = 1 << 0,
	CONFIG_DIFF_CREDENTIALS      = 1 << 1,
	CONFIG_DIFF_AGENT_POLICY     = 1 << 2,
	CONFIG_DIFF_VLAN             = 1 << 3,
	CONFIG_DIFF_AGENT_POLICY_CNT = 1 << 4,
};

enum aptype {
	AP_WIFI_FBSS,
	AP_WIFI_BBSS,
	AP_WIFI_COMBINED,
};

enum wifi_freqband {
	BAND_2       = 1 << 0,        /**< 0x1 for 2.4Ghz band. */
	BAND_5       = 1 << 1,        /**< 0x2 for 5Ghz band. */
	BAND_DUAL    = 3,             /**< 0x3 for both 2.4 and 5Ghz bands */
	BAND_60      = 1 << 2,        /**< 0x4 for 60Ghz */
	BAND_6       = 1 << 3,        /**< 0x8 for 6Ghz */
	BAND_UNKNOWN = 1 << 4,        /**< 0x10 (and above) for unknown band */
};

enum wifi_security_type {
	WIFI_SECURITY_NONE,
	WIFI_SECURITY_WEP64,
	WIFI_SECURITY_WEP128,
	WIFI_SECURITY_WPAPSK,
	WIFI_SECURITY_WPA2PSK,
	WIFI_SECURITY_FT_WPA2PSK,
	WIFI_SECURITY_WPA3PSK,
	WIFI_SECURITY_WPA3PSK_T,
	WIFI_SECURITY_FT_WPA3PSK,
	WIFI_SECURITY_WPA,
	WIFI_SECURITY_WPA2,
	WIFI_SECURITY_WPA3,
};

struct stax {
	char macstring[32];     /* ':' separated mac address string */
	struct list_head list;
};

struct agent {
	uint8_t agent_id[6];     /* ':' separated mac address string */
	struct list_head list;
};

struct iface_credential {
	enum wifi_freqband band;
	enum wifi_security_type sec;
	uint8_t key[65];
	uint8_t bssid[6];
	uint8_t ssid[33];
	uint16_t vlanid;
	uint8_t multi_ap;
	enum aptype mode;
	uint8_t disallow_bsta;
	bool enabled;
	struct list_head list;

	/* wps attributes */
	char manufacturer[65];
	char model_name[33];
	char device_name[33];
	char model_number[33];
	char serial_number[33];
	uint8_t device_type[8];
};

enum agent_steer_policy {
	AGENT_STEER_DISALLOW,                /* agent shall not steer based on rcpi */
	AGENT_STEER_RCPI_MANDATE,            /* agent shall steer based on rcpi */
	AGENT_STEER_RCPI_ALLOW,              /* agent may steer based on rcpi */
};

enum backhaul_type {
	BK_TYPE_NONE
};

struct node_policy {
	uint8_t agent_id[6];
	uint8_t bk_ul_mac[6];
	uint8_t bk_dl_mac[6];
	enum backhaul_type type;
	uint8_t pvid;
	uint8_t pcp;
	bool report_scan;                     /* report independent scans */
	bool report_sta_assocfails;           /* report STA assoc fails */
	uint32_t report_sta_assocfails_rate;  /* reporting rate for STA assoc fails (attempts per minute) */
	uint8_t report_metric_periodic;       /* 0 = disable, else 1 - 255 in secs */
	bool steer_disallow;
	bool coordinated_cac;
	bool traffic_separation;

	int num_steer_stas;                   /* num of stas excluded from steering */
	int num_btmsteer_stas;                /* num of stas excluded from BTM steering */
	bool sta_steer;

	bool is_policy_diff;                  /* whether section changed when reloaded */

	struct list_head list;                /* attached to nodelist */
	/* custom policies follow */
	struct list_head radiolist;           /* list of configured radio sections */
	struct list_head steer_exlist;	      /* exclude stas from steering */
	struct list_head btmsteer_exlist;     /* exclude stas from BTM steering */
};

struct radio_policy {
	uint8_t macaddr[6];
	uint8_t agent_id[6];
	enum wifi_freqband band;              /* frequency band */
	enum agent_steer_policy policy;       /* 0, 1, 2 - see MultiAP specs */
	uint8_t util_threshold;               /* utilization as in BSS load IE */
	uint8_t rcpi_threshold;               /* 0 - 220 */
	uint8_t report_rcpi_threshold;        /* 0, or 1 - 220 */
	uint8_t report_util_threshold;        /* 0, or channel utilization value */
	uint8_t report_rcpi_hysteresis_margin;/* 0, or > 0 - hysteresis margin */
	bool include_sta_stats;               /* sta stats in AP metric response */
	bool include_sta_metric;              /* sta metric in AP metric response */

	struct list_head list;                /* link to next policy */
};

struct steer_control_config {
	char name[64];
	struct list_head list;
};

struct controller_config {
	bool enabled;
	bool has_registrar_5g;
	bool has_registrar_2g;
	int debug_level;
	int resend_num;
	bool enable_sta_steer;
	bool enable_bsta_steer;
	bool use_bcn_metrics;
	bool use_usta_metrics;
	int num_bss;
	int num_apolicy;
	int acs_timeout;
	int dfs_cleanup_timeout;
	struct list_head nodelist;
	struct list_head aplist;
	struct list_head radiolist;
	struct list_head sclist;		/* steer-control list */
};

struct controller;

int set_value_by_string(const char *package, const char *section,
		const char *key, const char *value, enum uci_option_type type);
int cntlr_config_add_agent(struct controller_config *c, char *al_mac);
int cntlr_config_add_agent_radio(struct controller_config *c, char *al_mac,
		char *radio_mac, char *band);
bool uci_set_option(char *package_name, char *section_type,
		char *search_key, char *search_val,
		char *option, char *value);

uint8_t cntlr_config_reload(struct controller_config *cfg);
int cntlr_config_defaults(struct controller *c, struct controller_config *cfg);
void cntlr_config_dump(struct controller_config *cfg);
int cntlr_config_clean(struct controller_config *cfg);
#endif
